//Utility class for sorting and searching arrays of integers.
class Utility {
  static void sortBySelection(int[] numbers) {
    // For each pass:
    for (int next=0; next < numbers.length-1; next++) { // (1) outer loop
      // Find the smallest value in the unsorted part of the array
      // Note that we need to find the position of the smallest value
      // in order to swap values later.
      int smallest = next;
      for (int current = next+1;
           current < numbers.length;
           current++) {                                 // (2) inner loop
        if (numbers[current] < numbers[smallest]) {
          smallest = current;
        }
      }

      assert smallest > next && smallest < numbers.length :
        "Index for smallest value " + smallest +
        " should have been between " + (next+1) +
        " and " + numbers.length;

      // Swap the smallest value with the first element
      // of the unsorted part
      int temp = numbers[smallest];                     // (3)
      numbers[smallest] = numbers[next];
      numbers[next] = temp;

      printArray("Pass #" + (next+1) + ": ", numbers);
    }
  }

  static void sortByInsertion(int[] numbers) {
    // For each pass:
    for (int next = 1; next < numbers.length; next++) {// (1) outer loop
      // Store the next element to be inserted in an auxilliary variable
      int temp = numbers[next];
      // Pass through the sorted subarray backwards
      //   If the current element is larger than the next element
      //     Shift the current element one position towards the end of the array
      int current;
      for (current = next;
           current > 0 && temp < numbers[current-1];
           current--) {                            // (2) inner loop
        numbers[current] = numbers[current-1];     // (3) shift backwards
      }

      assert current == 0 || numbers[current] < temp :
        "Index for insertion " + current +
        " should have been between 0 and " + (numbers.length-1);

      // Insert the next element in the vacant position in the array
      numbers[current] = temp;                     // (4)

      printArray("Pass #" + next + ": ", numbers);
    }
  }

  static int binarySearch(int[] numbers,  int key) {
    int lower = 0;                 // Lower limit of the search interval
    int upper = numbers.length-1;  // Upper limit of the search interval
    int middle = -1;               // Index of middle element
    boolean found = false;         // Indicates whether the key is found
    boolean moreElements = true;   // Indicates whether there are
                                   // more elements to search

    // Repeat while more elements in array:
    while (moreElements && !found) { // (1) Main loop
      middle = (lower + upper)/2;    // (2) Middle element in this interval
      if (lower > upper) {           // (3) Determine if interval is empty
        moreElements = false;        // No more elements left to search
      } else if (numbers[middle] == key) { // Key is in the middle element
        found = true;
      } else if (numbers[middle] < key) {
        lower = middle + 1;          // (4) Search in upper half
      } else {
        upper = middle - 1;          // (5) Search in lower half
      }
    }
    assert (!moreElements || found) : "Error in binary search.";

    if (found) {
      return middle;  // The key is in element numbers[middle]
    } else {
      return -1;      // The key is not found
    }
  }

  static int linearSearch(int[] numbers,  int key) {
    boolean found = false;
    int i = 0;
    // Repeat while more elements in array:
    while (i < numbers.length && !found) {  // (1)
      // If the current element contains the key:
      //   Return the index of the current element
      if (numbers[i] == key) {
        found = true;       // Found!
      } else {
        i++;                // Move to next element
      }
    }
    assert (i >= numbers.length || found) : "Error in linear search.";
    if (found)
      return i;             // The key is in element numbers[i]
    else
      return -1;            // The key is not found
  }

  static void printArray(String prompt, int[] array) {
    System.out.print(prompt);
    for (int value : array) {
      System.out.print(value + " ");
    }
    System.out.println();
  }

  // Methods for the Programming Exercises.
  static void sortBySelectionDescending(int[] numbers) {
    // For each pass:
    for (int next=0; next<numbers.length-1; next++) { // (1) outer loop
      // Find the largest value in the unsorted part of the array
      int largest = next;
      for (int current=next+1; current<numbers.length; current++) {
        // (2) inner loop
        if (numbers[current] > numbers[largest]) {
          largest = current;
        }
      }

      assert largest >= 0 && largest <= next :
        "Index for largest value " + largest
        + " should have beeen between " + 0 + " and " + next;

      // Swap the smallest value with the first element of the usorted part
      int temp = numbers[largest];                    // (3)
      numbers[largest] = numbers[next];
      numbers[next] = temp;

      printArray("Pass #" + (next+1) + ": ", numbers);
    }
  }
}